#include <string.h>
#include <dos.h>

#include <coninout.h>
#include <desqview.h>

int MAXY = 24;
int MINY = 1;

#include "ansi.h"

int ansiout::put( char ch )
{
	// Pass all non-ANSI sequences straight through to screen
	if( ch != 27 && ch != 31 && *str == 0 )
		return tty( ch );

	// Interpret ESC character
	if( ch == 27 )
	{
		// Invalid ANSI sequences go straight through
		if( *str != 0 )
		{
			ANSIwrite( str );
			tty( ch );
			*str = 0;
			str_len = 0;
		}
		// otherwise, start a new ANSI sequence
		else
		{
			*str = 27;
			str[1] = 0;
			str_len = 1;
		}
		return ch;
	}

	if( *str == 27 && str_len == 1 )
	{
		switch( ch )
		{
			case '[':
				// ANSI sequences must have a '['
				str[ str_len++ ] = '[';
				str[ str_len ] = 0;
				break;
			default:
				// Invalid ANSI sequence
				ANSIwrite( str );
				tty( ch );
				*str = 0;
				str_len = 0;
		}
		return ch;
	}

	if( ch == '[' && str_len > 0 )
	{
		// Invalid sequence
		ANSIwrite( str );
		tty( '[' );
		*str = 0;
		str_len = 0;
		return ch;
	}

	if( strchr( "0123456789;ABCDfHJKmsu", ch ) == NULL )
	{
		// Invalid sequence
		ANSIwrite( str );
		tty( ch );
		*str = 0;
		str_len = 0;
		return ch;
	}

	if( strchr( "ABCDfHJKmsu", ch ) != NULL )
	{
		interpretpos = 2;

		switch( ch )
		{
			case 'A':
				p = param();
				if( p == -1 )	p = 1;
                                if( gety() - p < 1 )
                                        gotoxy( getx(), 1 );
				else
                                        gotoxy( getx(), gety() - p );
				break;
			case 'B':
				p = param();
				if( p == -1 )	p = 1;
                                if( getx() + p > MAXY )
                                        gotoxy( getx(), MAXY );
				else	
                                        gotoxy( getx(), gety() + p );
				break;
			case 'C':
				p = param();
				if( p == -1 )	p = 1;
                                if( getx() + p > 80 )
                                        gotoxy( 80, gety() );
				else
                                        gotoxy( getx() + p, gety() );
				break;
			case 'D':
				p = param();
				if( p == -1 )	p = 1;
                                if( getx() - p < 1 )
                                        gotoxy( 1, gety() );
				else   
                                        gotoxy( getx() - p, gety() );
				break;
			case 'H':
			case 'f':
				y = param();
				x = param();
				if( y < 1 )	y = 1;
				if( x < 1 )	x = 1;
				if( x > 80 || x < 1 || y > MAXY || y < 1 )
				{
					*str = 0;
					str_len = 0;
					return 0;
				}
				gotoxy( x, y );
				break;
			case 'J':
				p = param();
				if( p == 2 )	
                                        clrscr();
				if( p == 0 )
				{
                                        x = getx();
                                        y = getx();
                                        clrscr( y, ( MAXY - MINY + 1 ) );
                                        gotoxy( x, y );
				}
				if( p == 1 )
				{
                                        x = getx();
                                        y = gety();
                                        clrscr( 1, y );
                                        gotoxy( x, y );
				}
				break;
			case 'K':
				sout.ClearEOL();
				break;
			case 'm':
				if( !strcmp( str, "\033[" ) )
				{
					*str = 0;
					str_len = 0;
					col = 0x07;
				}
				do
				{
					p = param();
					switch( p )
					{
						case -1:
							break;
						case 0:
							col = 0x07;
							break;
						case 1:
							col |= 0x08;
							break;
						case 5:
							col |= 0x80;
							break;
						case 7:
							col = ( (col&15) << 4 ) | ( col >> 4 );
							break;
						case 30:
						case 31:
						case 32:
						case 33:
						case 34:
						case 35:
						case 36:
						case 37:
							p -= 30;
							col = ( col & 0xf8 ) |
							      ( ( p & 4 ) >> 2 ) |
							      ( p & 2 ) |
							      ( ( p & 1 ) << 2 );
							break;
						case 40:
						case 41:
						case 42:
						case 43:
						case 44:
						case 45:
						case 46:
						case 47:
							p -= 40;
							col = ( col & 0x8f ) |
							      ( ( p & 4 ) << 2 ) |
							      ( ( p & 2 ) << 4 ) |
							      ( ( p & 1 ) << 6 );
					}
				} while( p != -1 );
				sout.SetAttr( col );
				break;
			case 's':
                                savey = gety();
                                savex = getx();
				break;
			case 'u':
				if( savey > 0 && savex > 0 )
					gotoxy( savex, savey );
				break;
		}
		*str = 0;
		str_len = 0;
		return ch;
	}

	if( ( ch == ';' ) || ( ch >= '0' && ch <= '9' ) )
	{
		str[ str_len++ ] = ch;
		str[ str_len ] = 0;
	}

	if( str_len > 50 )
	{
		ANSIwrite( str );
		*str = 0;
		str_len = 0;
		return 0;
	}

	return ch;
}

int ansiout::getx() { return sout.WhereX(); }
int ansiout::gety() { return sout.WhereY(); }
void ansiout::gotoxy( int x, int y ) { sout.GotoXY( x, y ); }

void ansiout::clrscr( int Y1, int Y2 )
{
	if( Y1 == -1 )
	{
		sout.Clear();
		return;
	}

	int x1, y1, x2, y2;
	byte x, y;

	sout.GetRegion( x1, y1, x2, y2 );
	x = sout.WhereX();
	y = sout.WhereY();
	sout.Region( x1, Y1, x2, Y2 );
	sout.Clear();
	sout.Region( x1, y1, x2, y2 );
	sout.GotoXY( x, y );
}

void ansiout::tabulate()
{
	int x;

	x = sout.WhereX();
	if( x < 80 )
	{
		do
			x++;
		while( x % 8 );
	}
	if( x == 80 )
		x = 1;
	sout.GotoXY( x, sout.WhereY() );
	if( x == 1 )
		sout << "\r\n";
}

void ansiout::backspace(void)
{
	if( sout.WhereX() > 1 )
		sout << '\b';         // non-destructive backspace
	else if( sout.WhereY() > 1 )
	{
		sout.GotoXY( 80, sout.WhereY() - 1 );
		sout << ' ';
		sout.GotoXY( 80, sout.WhereY() - 1 );
	}
}

char ansiout::tty( char ch )
{
	switch( ch )
	{
		case 7:
			sound( 2000 );
			TimedDelay( 75 );
			nosound();
			break;
		case 8:
			backspace();
			break;
		case 9:
			tabulate();
			break;
		case 11:
			sout.GotoXY( 1, 1 );
			break;
		case 12:
			sout.Clear();
			break;
		case 27:
			break;
		default:
                        sout << ch;
       }
       return ch;
}

void ansiout::ANSIwrite( char *s )
{
	while( *s )
		tty( *s++ );
}

int ansiout::param()
{
	int x, xx, b;

	for( b = 0, x = interpretpos; x < str_len; x++ )
		if( str[x] >= '0' && str[x] <= '9' )
			b = 1;

	if( !b )	
		return -1;

	if( str[interpretpos] == ';' )
	{
		interpretpos++;
		return 0;
	}

	xx = 0;
	do
	{
		xx = xx * 10 + ( str[interpretpos++] - '0' );
	} while( ( str[interpretpos] >= '0' && str[interpretpos] <= '9' ) && xx < 100 && interpretpos < str_len );

	if( xx >= 100 )
	{
		ANSIwrite( str );
		*str = 0;
		str_len = 0;
		return -1;
	}

	if( str[interpretpos] == ';' )
		interpretpos++;

	return xx;
}

ansiout::ansiout()
{
	*str = 0;
	str_len = 0;
	savex = savey = -1;
	col = 0x07;

	if( DetectDESQview() || DetectMWindows() )
		conout::BIOSVideo();
}

ansiout::~ansiout()
{
        sout.Cursor();
        sout.NoRegion();
        sout.GotoXY( 1, aout.Height );
        sout.ClearEOL();
        sout.GotoXY( 1, aout.Height-1 );
}

ansiout aout;

void ansi_init()
{
        extern byte screen_cols;
        extern byte screen_rows;

        aout.Width = screen_cols;
        aout.Height = screen_rows;
        if( aout.Height < 12 )       aout.Height = 25;
        if( aout.Width < 80 )        aout.Width = 80;

        MINY = 1;
        MAXY = aout.Height-1;

        sout.Clear();
        sout.Region( 1, 2, aout.Width, aout.Height-1 );
	sout.Cursor( 0, 255 );
}

