#include <bios.h>
#include <stdlib.h>
#include <conio.h>
#include <inout.h>
#include "coninout.h"

conout sout;
conin sin;

int conout::X1 = 1;
int conout::X2 = 80;
int conout::Y1 = 1;
int conout::Y2 = 25;

conout::conout(): standardout( 0 )
{
	GetRegion( X1, Y1, X2, Y2 );
}

conout::conout( int w ): standardout( w )
{
	GetRegion( X1, Y1, X2, Y2 );
}

conout::~conout()
{
}

char CC conout::_put( char c )
{
	if( WhereX() > X2-X1+1 && c != '\r' && c != '\n' && c != '\b' && c != '\a' )
	{
		putch( '\r' );
		putch( '\n' );
	}

	putch( c );
	return c;
}

void CC conout::changecolor()
{
}

conin::conin( conout& c ): wout(c)
{
        extendedkey = 0;
}

conin::~conin()
{
}

void CC conin::begininput()
{
}

void CC conin::endinput()
{
}

int CC conin::keyhit()
{
	return bioskey( 1 ) != 0;
}

unsigned conin::GetKey()
{
	extendedkey = 0;
	unsigned x = bioskey( 0 );
        if( x & 0xff )
                return x & 0xff;
        return x;
}

char CC conin::get()
{
        if( extendedkey )
        {
		int x = extendedkey;
                extendedkey = 0;
                return x;
        }
	if( !wout.RawMode() )
		wout << "\b";
        unsigned c = GetKey();
	if( !wout.RawMode() )
		wout << " \b";
	if( ( c >> 8 ) == 0 ) return c;
        if( ( c >> 8 ) == 1 ) return c & 0xff;
        extendedkey = c >> 8;
	return 0;
}

char CC conin::out( char c )
{
	wout << c;
	return c;
}

void CC conout::SetAttr( int f, int b )
{
        if( b == -1 ) b = current_back;
        beforeout();
        textattr( ( b << 4 ) | f );
	afterout();
        new_fore = current_fore = f;
        new_back = current_back = b;
}

conout& conout::GotoXY( int x, int y )
{
	beforeout();
	gotoxy( x, y );
	afterout();
	return *this;
}

static int currentcursor = _NORMALCURSOR;

void conout::Cursor()
{
        _setcursortype( currentcursor = _NORMALCURSOR );
}

void conout::Cursor( byte start, byte )
{
	if( start < 100 )
                _setcursortype( currentcursor = _SOLIDCURSOR );
	else
                _setcursortype( currentcursor = _NORMALCURSOR );
}

void conout::NoCursor()
{
        _setcursortype( currentcursor = _NOCURSOR );
}

void conout::GetRegion( int& x1, int& y1, int& x2, int& y2 )
{
	text_info t;
	gettextinfo( &t );

	X1 = x1 = t.winleft;
	Y1 = y1 = t.wintop;
	X2 = x2 = t.winright;
	Y2 = y2 = t.winbottom;
}

void conout::Region( int x1, int y1, int x2, int y2 )
{
        X1 = x1;
        Y1 = y1;
        X2 = x2;
        Y2 = y2;
        window( x1, y1, x2, y2 );
}

void conout::NoRegion()
{
	text_info t;
	gettextinfo( &t );

        X1 = 1;
        Y1 = 1;
        X2 = t.screenwidth;
        Y2 = t.screenheight;
        window( 1, 1, t.screenwidth, t.screenheight );
}

void conout::ScrollUp( int dy )
{
	byte x, y;
        x = sout.WhereX();
        y = sout.WhereY();
        sout.GotoXY( 1, 1 );
	while( dy-- > 0 )
	{
		delline();
		if( y > 1 ) y--;
	}
        sout.GotoXY( x, y );
}

void conout::ScrollDown( int dy )
{
	byte x, y;
	int x1, y1, x2, y2;
        x = sout.WhereX();
        y = sout.WhereY();
	GetRegion( x1, y1, x2, y2 );
        sout.GotoXY( 1, 1 );
	while( dy-- > 0 )
	{
		insline();
		if( y < y2-y1+1 ) y++;
	}
        sout.GotoXY( x, y );
}

conout& conout::Clear()
{
	beforeout();
	gotoxy( 1, 1 );
	clrscr();
	afterout();
	return *this;
}

conout& conout::ClearEOL()
{
	beforeout();
	clreol();
	afterout();
	return *this;
}

int conout::WhereX()
{
	return ::wherex();
}

int conout::WhereY()
{
	return ::wherey();
}

int conout::GetAttr()
{
	return current_fore | ( current_back << 4 );
}

void conout::BIOSVideo()
{
	directvideo = 0;
}

void conout::DirectVideo()
{
	directvideo = 1;
}

void conout::SetVideoBorder( int )
{
        // not implemented
}

SaveScreen::SaveScreen( int pr )
{
	if( pr )
		buf = 0;
	else
		save();
}

void SaveScreen::save()
{
	gettextinfo( &t );

	buf = new char[ 2 * t.screenheight * t.screenwidth ];
	if( buf )
		gettext( 1, 1, t.screenwidth, t.screenheight, buf );
	_setcursortype( _NOCURSOR );
}

SaveScreen::~SaveScreen()
{
	restore();
}

void SaveScreen::restore()
{
	if( buf )
	{
		puttext( 1, 1, t.screenwidth, t.screenheight, buf );
		delete buf;
		buf = 0;
	}

	sout.Region( t.winleft, t.wintop, t.winright, t.winbottom );
	sout.GotoXY( t.curx, t.cury );
	sout.SetAttr( t.attribute );
	_setcursortype( currentcursor );
}

static text_info scrstate;
byte screen_cols = 80;
byte screen_rows = 25;

void conio_exit()
{
	window( 1, scrstate.screenheight, scrstate.screenwidth, scrstate.screenheight );
	textattr( 0x07 );
	clrscr();
}

void conio_init()
{
	static int init = 0;
	if( init ) return;
	init = 1;
	gettextinfo( &scrstate );
	screen_cols = scrstate.screenwidth;
	screen_rows = scrstate.screenheight;
	directvideo = 0;
	textattr( 0x07 );
	atexit( conio_exit );
}

#pragma startup conio_init 80

// replacement getch() that does it better!  :)
extern "C" int getch()
{
	char c;
	sin >> c;
	return c;
}
