#include <dos.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>

#include <inout.h>
#include <datetime.h>

static struct time unix2time( long x )
{
	struct time t;
	struct date d;
	unixtodos( x, &d, &t );
	return t;
}

static struct date unix2date( long x )
{
	struct time t;
	struct date d;
	unixtodos( x, &d, &t );
	return d;
}

Time::Time()
{
	time( &unixdate );
}

long Time::Second() const
{
	return unix2time(unixdate).ti_sec;
}

long Time::Minute() const
{
	return unix2time(unixdate).ti_min;
}

long Time::Hour() const
{
	return unix2time(unixdate).ti_hour;
}

long Time::Day() const
{
	return unix2date(unixdate).da_day;
}

long Time::Month() const
{
	return unix2date(unixdate).da_mon;
}

long Time::Year() const
{
	return unix2date(unixdate).da_year;
}

long operator - ( Time a, Time b )
{
	return a.unixdate - b.unixdate;
}

Time operator + ( Time a, long b )
{
        return Time(a.Clock()+b);
}

Time DateOnly( Time x )
{
	struct time t;
	struct date d;
	unixtodos( x.unixdate, &d, &t );
	t.ti_hund = 0;
	t.ti_sec = 0;
	t.ti_min = 0;
	t.ti_hour = 0;
	return Time( dostounix( &d, &t ) );
}

standardout& operator << ( standardout& out, Time t )
{
	return out << DateDisplay( t );
}

const char * DateDisplay( Time t )
{
	struct date dd;
	struct time tt;

	unixtodos( t.unixdate, &dd, &tt );

	static char s[40];
	s[0] = '\0';
	itoa( dd.da_day, s, 10 );
	strcat( s, "-" );

	const char* mon[] = { "", "Jan", "Feb", "Mar", "Apr", "May", "Jun",
				  "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
	strcat( s, mon[dd.da_mon] );
	strcat( s, "-" );
	itoa( dd.da_year, s+strlen(s), 10 );
	if( tt.ti_hour != 0 || tt.ti_min != 0 || tt.ti_sec != 0 || tt.ti_hund != 0 )
	{
		strcat( s, " " );
		int i = tt.ti_hour;
		if( i >= 12 ) i -= 12;
		if( i == 0 ) i = 12;
		itoa( i, s+strlen(s), 10 );

		i = strlen(s);
		s[i++] = ':';
		s[i++] = '0' + tt.ti_min / 10;
		s[i++] = '0' + tt.ti_min % 10;
//              s[i++] = ':';
//              s[i++] = '0' + tt.ti_sec / 10;
//              s[i++] = '0' + tt.ti_sec % 10;
//              s[i++] = '.';
//              s[i++] = '0' + tt.ti_hund / 10;
//              s[i++] = '0' + tt.ti_hund % 10;
		if( tt.ti_hour < 12 )
			s[i++] = 'a';
		else
			s[i++] = 'p';
		s[i++] = 'm';
		s[i] = '\0';
	}

	return s;
}

Time Now;
Time Today = DateOnly(Now);

