#include <time.h>
#include <dos.h>

static enum { None, DESQview, MWindows } Multitasker = MWindows;

unsigned DetectDESQview()
{
	union REGS r;
	r.x.cx = 0x4445;
	r.x.dx = 0x5351;
	r.x.ax = 0x2b01;
	intdos( &r, &r );
	if( r.h.al != 0xff )
		// high/low byte = version of DESQview
		return r.x.bx;

        // No DESQview
	return 0;
}

void DESQviewTimeSlice()
{
	// Save AX
	asm push ax;

	// Switch to DESQview's stack
	asm mov ax, 0x101A;
	asm int 0x15;

	// Give up time slice
	asm mov ax, 0x1000;
	asm int 0x15;

	// Switch back from DESQview's stack
	asm mov ax, 0x1025;
	asm int 0x15;

	// Restore AX
	asm pop ax;
}

void DESQviewBeginCritical()
{
	// Save AX
	asm push ax;

	// Switch to DESQview's stack
	asm mov ax, 0x101A;
	asm int 0x15;

	// Begin Critical section
	asm mov ax, 0x101B;
	asm int 0x15;

	// Switch back from DESQview's stack
	asm mov ax, 0x1025;
	asm int 0x15;

	// Restore AX
	asm pop ax;
}

void DESQviewEndCritical()
{
	// Save AX
	asm push ax;

	// Switch to DESQview's stack
	asm mov ax, 0x101A;
	asm int 0x15;

	// End Critical section
	asm mov ax, 0x101C;
	asm int 0x15;

	// Switch back from DESQview's stack
	asm mov ax, 0x1025;
	asm int 0x15;

	// Restore AX
	asm pop ax;
}

unsigned DetectMWindows()
{
	union REGS r;
	r.x.ax = 0x1600;
	int86( 0x2F, &r, &r );
	if( r.h.al == 0x00 || r.h.al == 0x80 )
		// Not running MS-Windows
		return 0;

	if( r.h.al == 0x01 || r.h.al == 0xff )
		// MS-Windows/386 version 2.xx
		return 0x200;

	// MS-Windows version AL.AH
	return ( r.h.al << 16 ) | r.h.ah;
}

void MWindowsTimeSlice()
{
	// Give up time slice
	asm mov ax, 0x1680;
	asm int 0x2F;
        asm mov ax, 0x1680;
        asm int 0x2F;
        asm mov ax, 0x1680;
        asm int 0x2F;
}

void MWindowsBeginCritical()
{
	// Begin Critical section
	asm mov ax, 0x1681;
	asm int 0x2F;
}

void MWindowsEndCritical()
{
	// End Critical section
	asm mov ax, 0x1682;
	asm int 0x2F;
}

void _DetectStartupMultitasker()
{
	if( DetectDESQview() )
		Multitasker = DESQview;
	else if( DetectMWindows() )
		Multitasker = MWindows;
	else
		Multitasker = None;
}

#pragma startup _DetectStartupMultitasker

void TimeSlice()
{
        if( Multitasker == DESQview )   DESQviewTimeSlice();
	if( Multitasker == MWindows )   MWindowsTimeSlice();
//        asm int 0x28;
}

void BeginCritical()
{
	if( Multitasker == DESQview )   DESQviewBeginCritical();
	if( Multitasker == MWindows )   MWindowsBeginCritical();
}

void EndCritical()
{
	if( Multitasker == DESQview )   DESQviewEndCritical();
	if( Multitasker == MWindows )   MWindowsEndCritical();
}

void TimedDelay( unsigned ms )
{
#ifndef bug
	long start = clock();
	while( start + ms * 182l / 10000l > clock() )
		TimeSlice();
#else
	delay( ms );
#endif
}

void DisableMultitasker()
{
	Multitasker = None;
}

void EnableMultitasker()
{
        Multitasker = MWindows;
}

