#pragma hdrstop
#include <time.h>
#include <dos.h>
#include <mem.h>
#include <ctype.h>
#include <conio.h>
#include <stdlib.h>

#include <ansi.h>
#include <fossil.h>

#include <types.h>
#include <door.h>
#include <srinout.h>
#include <finout.h>

#include "\sr\srdoor.h"
#include <srparam.h>

// This system requires a function be created with this header that
// returns strings; it must return "~ERROR" for invalid indices; if
// you do not want this feature, just define the function to always
// reutrn "~ERROR";.
const char *itemdescription( int index );

// DOORFILE.SR includes:
//   Username
//   Ansi     ( 1 = yes, 0 = no, -1 = ask )
//   IBMChar  ( 1 = yes, 0 = no, -1 = same as ANSI )
//   PageLen  ( #, or 0 = default )
//   BaudRate ( #, or 0 = default )
//   ComPort  ( #, or 0 = Local )
//   TimeLimit( #, or -1 = no limit )

DoorInfo DOOR;
const char *DoorInfo::DoorFile = "DOORFILE.SR";

DoorInfo::DoorInfo()
{
        *name = 0;
        ansi = 1;
        ibmcharacters = 1;
        pagelength = 25;
        baudrate = 2400;
        comport = 0;
        timelimit = -1;
        loaded = 0;
}

void DoorInfo::init()
{
	filein f( DoorFile );

        if( !!f )
        {
                char s[256];
                long l;

                f >> name;
                f >> s;         ansi = atoi(s);
                f >> s;         ibmcharacters = atoi(s);
                f >> s;         pagelength = atoi(s);
                f >> s;         l = atol(s);
                                baudrate = (unsigned)( l > 65535u ? 38400u : l );
                f >> s;         comport = atoi(s);
                f >> s;         l = atol(s);
                                timelimit = (int)( l > 32767u ? 32767u : l );
        }

        if( !!f )
                loaded = 1;

	realbaudrate = baudrate;

	Resource rAnsi( "Ansi", ansi );
	Resource rIBMCharacters( "IBMCharacters", ibmcharacters );
        Resource rCheckTimeLimit( "CheckTimeLimit", 1L );
        Resource rLocalPlay( "Local", 0L );
        Resource rLockBaud( "LockBaudRate", 0L );
        Resource rTimeOut( "TimeOut", NoKeyTimeOut );

        NoKeyTimeOut = (int)( rTimeOut.Long() );
        if( rAnsi.Long() != -1 )
		ansi = rAnsi.Boolean();
	if( rIBMCharacters.Long() != -1 )
		ibmcharacters = rIBMCharacters.Boolean();
	if( !rCheckTimeLimit.Boolean() )
		timelimit = -1;
	if( rLocalPlay.Boolean() )
		comport = 0;
	if( rLockBaud.Long() != 0 )
	{
		long l = rLockBaud.Long();

		if( l >= 300 && l <= 65535u )
                {
			baudrate = (unsigned)l;
			local << "Locking baud rate at " << attr(11) << baudrate << attr() << " baud.\n";
                }
		else
                {
                        const char *s = "Invalid resource 'LockBaudRate':  baud rate must be from 300 to 65535 baud.\n";
//                        ErrorLog << s;
                        local << attr(14) << s << attr();
                }
	}

	::Ansi = ansi;
	::IBMCharacters = ibmcharacters;
}

DoorInfo::~DoorInfo()
{
}

int DoorInfo::read()
{
        if( !loaded )
        {
                const char *errormsg = " not found.  Please run the install program!\n";

		kout << DoorFile << errormsg;
  //              ErrorLog << DoorFile << errormsg;

                exit(1);
	}

	if( pagelength == 0 || pagelength < 10 )
		pagelength = 23;

        // Load global variables
        ::ComPort = comport;

        if( comport == 0 )
	{
		if( ansi == -1 )
			ansi = 1;
	}
	else
	{
		char c;

                fossil.init( comport, baudrate );

#if 0
		if( ansi == -1 )
			kout << "Do you have ANSI color?  [Y/n] ";
		else
			kout << "Press <SPACE> ";

		kout.SetRawMode();
		do
		{
                        kin >> c;
                        c = toupper( c );
			if( c == '\r' ) c = 'Y';
		} while( c != 'Y' && c != 'N' && c != ' ' && (unsigned char)c != 127 );
		kout.ResetRawMode();

		if( ansi == -1 )
			ansi = ( c == 'Y' );
		else
			ansi &= 1;
		if( (unsigned char)c == 127 )
		{
                        ansi = ibmcharacters = 1;
			statusbar( "SR-Term", "Loading Strings", '>' );

                        kout << "$START\n";

			// Send all the strings over to the remote side
			for( int i = 0; i < 256; i++ )
				if( *(itemdescription( i )) != '~' )
				{
					kout << '$' << i << '\n';
					kout << itemdescription( i );
                                        kout << '\n';  // End-of-string
				}
                        kout << "$END\n";  // End of all strings

                        statusbar( "SR-Term", "Ready", '!' );

			SRterminal = 1;
		}
#else
		if( ansi == -1 )
		{
			kout << "Do you have ANSI color?  [Y/n] ";

			kout.SetRawMode();
			do
			{
				kin >> c;
				c = toupper( c );
				if( c == '\r' ) c = 'Y';
			} while( c != 'Y' && c != 'N' );
			kout.ResetRawMode();

			ansi = ( c == 'Y' );
		}
#endif
	}

	if( ibmcharacters == -1 )
		ibmcharacters = ansi;
	else
		ibmcharacters &= 1;

	// Load global variables
	::Ansi = ansi;
	::IBMCharacters = ibmcharacters;
	::ComPort = comport;

	if( ::Ansi )
		kout.ResetFilterANSIMode();
	else
		kout.SetFilterANSIMode();

	return 1;
}

int displayfile( const char *filename, int coded )
{
        filein f( filename, 1 );  // 1 == binary mode
        char c;
        int fg = 7, bg = 0;

        if( !f )        return 0;

        while( !!f )
        {
                if( kin.keyhit() )
                {
                        char a;
                        kin >> a;
                        if( a == ' ' )
                        {
                                kout << attr(4) << "\nAborted\n" << attr();
                                return 1;
                        }
                }

                f >> c;
                c ^= coded;
                if( !f )
                        break;
                if( c == '^' )
                {
                        f >> c;
                        c ^= coded;
                        if( c == '\\' )
                        {
                                f >> c;
                                c ^= coded;
                                if( c > '@' )   bg = c - 'A' + 10;
                                else            bg = c - '0';
                                f >> c;
                                c ^= coded;
                                if( c > '@' )   fg = c - 'A' + 10;
                                else            fg = c - '0';
                                if( bg != 0 )
                                        kout << attr( fg, bg );
                                else
                                        kout << attr( fg );
                        }
                        else
                                kout << '^' << c;
                }
                else
                        kout << c;
        }

	kout << attr();
        return 1;
}

void initdoor()
{
        DOOR.init();
}

#pragma startup initdoor

