#ifndef F_INOUT

#define F_INOUT

#include "inout.h"

#ifndef byte
        typedef unsigned char byte;
#endif

class FileSystem
{
public:
	typedef void (*_ErrorHandler)( const char *s );
	friend class fileout;
	friend class filein;
	friend class recordfile;

private:
	static _ErrorHandler ErrorHandler;
	static void HandleError( const char *msg, const char *arg );

public:
	static void InstallErrorHandler( _ErrorHandler f = 0 );
        static _ErrorHandler GetErrorHandler();
};

class fileout : public standardout
{
private:
        int handle, err, binary;
        int buf_len, buf_ptr;
        char *buf;

        void getbuffer();

public:
	fileout();
        fileout( const char *filename, int appendflag = 1, int binaryflag = 0, int wide = 0 );
        ~fileout();

        int open( const char *filename, int appendflag = 1, int binaryflag = 0 );
        int close();

	virtual char CC _put( char c );

        int flush();
        int exists() { return handle != -1; }
        int operator !();
};

class filein : public standardin
{
private:
        int handle, err, binary;
        int buf_len, buf_ptr, buf_overflow;
        char *buf;

        void getbuffer();

public:
        filein();
        filein( const char *filename, int binaryflag = 0 );
        ~filein();

        int open( const char *filename, int binaryflag = 0 );
        int close();
        int exists() { return handle != -1; }

        virtual CC char get();
        virtual CC char out( char c );
        virtual CC void begininput();
        virtual CC void endinput();
        virtual CC int keyhit();

        int operator !();
};

class recordfile
{
private:
        char pathname[128];
        int handle;
        int size;
	int err;

	// Optimization --- where was the last accessed file position?
	long lastaccess;

        // encoding available for files
        byte xor_code, cycle_code;

        void encode( char *data, int count = 1 );

public:
	recordfile( const char *filename = "", int s = 1, byte new_code = 0, byte new_cycle = 1 );
	~recordfile();

	int open( const char *filename );
        int close();

        int CC write( long n, void *data, int count = 1 );
        int CC read( long n, void *data, int count = 1 );

        int CC erase_file();

	int lockrecord( long n );
	int unlockrecord( long n );

        long CC filesize();
	const char * CC filename() { return pathname; }
        int CC isopen() { return handle != -1; }
        int CC exists() { return handle != -1; }
        int CC error();
};

#endif

