#ifndef INOUT
#define INOUT

#define CC pascal

#include <types.h>
#include "longnumb.h"

class standardout;
class LineBuffer
{
private:
	char *buf;
        int *fg;
        int *bg;
	int bufpos;
	int linewidth;
	int bufsize;

	void copy( const LineBuffer& );
	void kill();

public:
	LineBuffer( int w );
	LineBuffer( const LineBuffer& b );
	~LineBuffer();

	LineBuffer& operator = ( const LineBuffer& );

	LineBuffer& insert( char c, int f, int b );
        LineBuffer& narrow( int w );
	LineBuffer& backspace();

	void clear();
	char operator [] ( int index );
	int fore( int index );
	int back( int index );

	int pos() { return bufpos; }
	int width() { return linewidth; }

	friend standardout& operator << ( standardout& out, const LineBuffer& b );
};

class standardin
{
private:
	int fieldwidth;
	number num_default;
	number num_max;

public:
	standardin();

        standardin& CC setdefault( number def = -1 );
        standardin& CC setmax( number max = -1 );

        int CC width();
        standardin& CC setw( int w = 0 );

        virtual char CC out( char c );
        virtual int CC keyhit() = 0;
        virtual char CC get() = 0;
        virtual void CC begininput();
        virtual void CC endinput();
        virtual void CC getstr( char * );
        virtual number CC readnum( number max );

        standardin& CC operator >> (char& c);
        standardin& CC operator >> (long& l);
        standardin& CC operator >> (char *s);
        standardin& CC operator >> (int& i);
        standardin& CC operator >> (unsigned& i);
        standardin& CC operator >> (unsigned char& i);
};

class standardout
{
private:
        LineBuffer buffer;
        int CommaFlag;
        int ESCflag;

protected:
        int fieldwidth;
        int rawmode;
	shortint current_fore, current_back;
	shortint new_fore, new_back;
        int FilterANSI;

public:
	void writenum( number );

        standardout( int newwidth = 0 );
        ~standardout();

        LineBuffer& CC narrow( int w )  { buffer.narrow( w ); return buffer; }
        LineBuffer CC Buffer()          { return buffer; }

        int CC width()                  { return fieldwidth; }

        void CC SetRawMode()            { rawmode = 1; }
        void CC ResetRawMode()          { rawmode = 0; }
        int CC RawMode()                { return rawmode; }

        void CC SetFilterANSIMode()     { FilterANSI = 1; }
        void CC ResetFilterANSIMode()   { FilterANSI = 0; }
        int CC FilterANSIMode()         { return FilterANSI; }

        void CC SetActualAttr( int f, int b = -1 ) { current_fore = f; current_back = b==-1? current_back : b; }
        virtual void CC SetAttr( int f, int b = -1 );
        int CC GetForeground() { return new_fore; }
        int CC GetBackground() { return new_back; }

        void CC SetCommaMode()          { CommaFlag = 1; }
        void CC ResetCommaMode()        { CommaFlag = 0; }
        int CC CommaMode()              { return CommaFlag; }

        virtual void CC beforeout();
        virtual void CC afterout();
        virtual void CC changecolor();

        virtual char CC put( char c );
        virtual char CC _put( char c );
        standardout& CC setw( int w = 0 );

        standardout& CC operator << (char c);
        standardout& CC operator << (long l);
        standardout& CC operator << (const char *s);
        standardout& CC operator << (int i);
        standardout& CC operator << (unsigned i);
        standardout& CC operator << (unsigned char i);
};

// Color object
struct attr
{
	// foreground and background data
	int fore;
	int back;

	// Default color is gray on black
	//   If b is -1, then the previous background is used (unchanged)
        CC attr( int f = 7, int b = 0 );

	// Output turns the attr into an ANSI sequence
        friend standardout& CC operator << ( standardout& out, attr& a );
};

typedef const char *( *FilterFunction )( const char* s );
FilterFunction SetTextFilter( FilterFunction filter );

#endif

