#define TRAP_MODE 1
// TRAP_MODE 0 = standard ^C is trapped, ^BREAK is not
// TRAP_MODE 1 = use signal() to trap both ^C & ^BREAK
//               also use an interrupt function for timeouts

#if TRAP_MODE
#include <dos.h>
#include <signal.h>
#endif

#include <bios.h>
#include <ctype.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>
#include <dos.h>

#include "fossil.h"
#include "ansi.h"
#include "desqview.h"
#include "inout.h"
#include "finout.h"
#include "srinout.h"
#include "srparam.h"

#define TimedDelay(n) delay(n)

Hook multitask_hook = 0;                // Multitasking code hook function
int NoKeyTimeOut = 60;                  // Time (seconds) before keyboard
                                        // timeout occurs

InputDevice LastRead = Keyboard;        // Last key read from Modem or Keyboard?

int Ansi = 0;				// Set to 1 if ansi screen
int ComPort = 0;			// 0 = local
                                        // 1..8 = com port #
int ExpertMode = 0;			// special key for programs to use
					// in their own way
int IBMCharacters = 1;			// IBM character set available

int SpecialEffects = 1;			// Pause whirly cursor
int LinesPrinted = 0;			// Number of lines printed
int PageLength = 21;                    // Page length

char BREAK_KEY = 3;                     // Control 'C' is break key
char X_BREAK_KEY = 0;                   // no extended break key

char STOP_KEY = 4;                      // Control 'D' is stop key
char X_STOP_KEY = 63;                   // F5 is the extended stop key

// Colors
attr aNormal( 7 );
attr aCursor( 15 );

// Standard keyboard-in/out object instances
keyin kin;
keyout kout;
Local local;
Remote remote;

struct _PauseColors PauseColor = { 0, 7, 15 };

fileout LogFile;

static int keybhit()
{
	return bioskey( 1 ) != 0;
}

static int getkey()
{
	int x = bioskey( 0 );
	if( ( x & 0xff ) != 0 )
		x = x & 0xff;
	return x;
}

/////////////////////////////////////////////////////////////////////keyintercept

keyintercept* keyintercept::head = 0;

keyintercept::keyintercept( Hook f, unsigned k, unsigned scan )
{
        anytime = 0;
        hook = f;
        next = head;
        head = this;
        key = scan * 256u + k;
}

keyintercept::~keyintercept()
{
        if( head == this )
        {
                head = next;
                return;
        }

        keyintercept* p = head;
        while( p && p -> next )
        {
                if( p -> next == this )
                {
                        p -> next = next;
                        return;
                }
                p = p -> next;
        }
}

Hook keyintercept::Search( unsigned k, int anytime )
{
        keyintercept* p = head;
        while( p )
        {
                if( p -> key == k )
                        if( p -> anytime || anytime )
                                return p -> hook;
                p = p -> next;
        }
        return 0;
}

Hook keyintercept::SearchExtended( unsigned scan, int anytime )
{
        return Search( scan * 256u, anytime );
}

/////////////////////////////////////////////////////////////////////keyintercept


/////////////////////////////////////////////////////////////////////keyin

void stop_key()
{
        #if !TRAP_MODE
                exit( 0 );
        #else
                raise( SIGTERM );
        #endif
}

void break_key()
{
        #if !TRAP_MODE
                exit( 1 );
        #else
                raise( SIGINT );
        #endif
}

keyin::keyin()
{
        frozen = 0;
}

keyin::~keyin()
{
}

char CC keyin::out( char c )
{
	kout << c;
	return c;
}

void CC keyin::begininput()
{

        savefore = kout.GetForeground();
	saveback = kout.GetBackground();
	kout << aCursor;
        if( aCursor.back > 0 && Ansi )
        {
                int rawmode = kout.RawMode();
                kout.SetRawMode();
                if( width() > 0 )
                {
                        for( int i = 0; i < width(); i++ )
                                kout << ' ';
                        if( Ansi )
                                kout << "\033[" << width() << 'D';
                        else
                                for( i = 0; i < width(); i++ )
                                        kout << '\b';
                }
                if( !rawmode ) kout.ResetRawMode();
        }
}

void CC keyin::endinput()
{
	kout << attr( savefore, saveback );
}

int multitask()
{
        // let DesqView or MS-Windows have more time
        TimeSlice();

        if( multitask_hook )
        {
                (*multitask_hook)();
                return 1;
        }
        return 0;
}

int CC keyin::keyhit()
{
        for(;;)
        {
                int predict = bioskey( 1 );
                if( predict == -1 )
			break_key();

                multitask();

                if( predict )
                {
                        Hook h = keyintercept::Search( predict );
                        if( h )
                        {
                                bioskey( 0 );
				(*h)();
			}
			predict = 0;
		}

		if( !fossil.carrierdetect() )
		{
			local << attr(14) << "Hangup?\n";
			exit(1);
		}

		if( !predict )
			return( !frozen && ( fossil.buffer() || ( keybhit() && kout.LocalOutput ) ) );
        }
}

static Resource UserBreak( "UserBreak", 1L );

char CC keyin::get()
{
	static int extendflag = 0;
        char promptchar = IBMCharacters? '' : '_';
        char* backspace = Ansi? "\033[D" : "\b";
        int rawmode = kout.RawMode();

	long t = 0;
	int ch;

        if( frozen )    return 0;

	LinesPrinted = 0;

	if( extendflag )
	{
                int t = extendflag;
                extendflag = 0;
                return t;
        }

        if( !rawmode )
        {
                kout.SetRawMode();
                kout << promptchar << backspace;
                kout.ResetRawMode();
        }

	time_t start_time, end_time;
	int warned = 0;

	time( &start_time );
	time( &end_time );

	for( ; start_time + 30 + NoKeyTimeOut > end_time; t++ )
	{
		if( ComPort != 0 && !fossil.carrierdetect() )
                        start_time--;

                if( keyhit() )
                {
                        if( ComPort != 0 && fossil.buffer() )
                        {
                                LastRead = Modem;

                                ch = fossil.read();
                                if( !rawmode && ch == 127 ) ch = '\b';
                                if( !rawmode ) kout << " \b";
                                if( ch == BREAK_KEY && UserBreak.Boolean() )
                                {
                                        LineBuffer oldline = kout.Buffer();
                                        int oldfore = kout.GetForeground();
                                        int oldback = kout.GetBackground();

                                        kout << attr() << "\nBreak out of Program?  [y/N]";
                                        for(;;)
                                        {
                                                char c;
                                                kin >> c;
                                                if( c == 'Y' || c == 'y' )
                                                        break_key();
                                                if( c == 'N' || c == 'n' || c == '\r' )
                                                        break;
                                        }

                                        if( !rawmode ) kout << attr() << oldline << attr(oldfore,oldback);
                                }
                                else if( ch == STOP_KEY && UserBreak.Boolean() )
                                {
                                        LineBuffer oldline = kout.Buffer();
                                        int oldfore = kout.GetForeground();
                                        int oldback = kout.GetBackground();

                                        kout << attr() << "\nStop Program?  [y/N]";
                                        for(;;)
                                        {
                                                char c;
                                                kin >> c;
                                                if( c == 'Y' || c == 'y' )
                                                        stop_key();
                                                if( c == 'N' || c == 'n' || c == '\r' )
                                                        break;
                                        }

                                        if( !rawmode ) kout << attr() << oldline << attr(oldfore,oldback);
                                }
                                else
                                        return ch;
                                if( !rawmode )
                                {
                                        kout.SetRawMode();
                                        kout << promptchar << backspace;
                                        kout.ResetRawMode();
                                }
                        }

			if( keybhit() )
                        {
                                LastRead = Keyboard;

				ch = getkey();

				if( ( ch & 0xff ) == 0 )
				{
					ch >>= 8;

					Hook h = keyintercept::SearchExtended( ch );
					if( h )
					{
						(*h)();
						continue;
					}

                                        if( rawmode ) kout << " \b";

					h = keyintercept::SearchExtended( ch, 1 );
                                        if( h )
                                                (*h)();
                                        else if( ch == X_BREAK_KEY )
                                                break_key();
                                        else if( ch == X_STOP_KEY )
                                                stop_key();
                                        else
                                        {
                                                extendflag = ch;
                                                return 0;
                                        }
                                }
				else
                                {
                                        Hook h = keyintercept::Search( ch );
                                        if( h )
                                        {
                                                (*h)();
                                                continue;
                                        }

                                        if( !rawmode ) kout << " \b";

                                        h = keyintercept::Search( ch, 1 );
                                        if( h )
						(*h)();
                                        else if( ch == BREAK_KEY )
                                                break_key();
                                        else if( ch == STOP_KEY )
                                                stop_key();
                                        else if( kout.LocalOutput )
                                                return ch;
                                }
                                if( !rawmode )
                                {
                                        kout.SetRawMode();
                                        kout << promptchar << backspace;
                                        kout.ResetRawMode();
                                }
                        }
                }

		time_t new_time;
		time( &new_time );
		if( new_time - end_time > 5 )
			// if 5 seconds have elasped, it *might* be paused
			// and we want to respect their wishes
			start_time += ( new_time - end_time );
		end_time = new_time;

		if( end_time - start_time > NoKeyTimeOut && !warned )
		{
			LineBuffer oldline = kout.Buffer();
                        int oldfore = kout.GetForeground();
                        int oldback = kout.GetBackground();

                        if( !rawmode )
			{
				kout << attr(14) << " \n\aWarning!  Program will abort in 30 seconds if you do not respond...\n";
                                kout << attr() << oldline << attr(oldfore,oldback);
			}
			else
				kout << "\a\a\a";
			warned = 1;
		}

		// Call the multi-tasking hook function if one exists
		multitask();
                delay( 5 );
	}

	if( ComPort != 0 )
		fossil.deinit();
	t = ComPort;
	ComPort = 0;
	kout << attr(14) << " \a\nTIMEOUT:  COM" << t << ": no input for " << attr(15) << (long)(end_time - start_time) << attr(14) << " seconds\n";
	exit( 2 );
	return 0;
}

/////////////////////////////////////////////////////////////////////keyout

keyout::keyout(): standardout( 79 )
{
        LogFileOpen = 0;
        LogFileOutput = 0;
        LocalOutput = 1;
        RemoteOutput = 1;
}

keyout::~keyout()
{
        if( LogFileOpen )       CloseLogFile();
}

void CC keyout::changecolor()
{
        current_fore = new_fore;
        current_back = new_back;
        if( LogFileOpen )
                LogFile << attr( new_fore, new_back );
        if( Ansi )
        {
                if( LocalOutput )
                        local << attr( new_fore, new_back );
                remote << attr( new_fore, new_back );
        }
}

void keyout::OpenLogFile( const char *filename )
{
        if( LogFileOpen )
                LogFile.close();
        LogFileOpen = 1;
        LogFile.open( filename, 0 );
	if( !Ansi ) LogFile.SetFilterANSIMode();
        LogFileOutput = 1;
}

void keyout::CloseLogFile()
{
	LogFile.close();
	LogFileOpen = 0;
        LogFileOutput = 0;
}

char CC keyout::_put( char c )
{
        int PauseFlag = 0;

        if( c == '\n' && !rawmode )
        {
                LinesPrinted++;

                if( LinesPrinted > PageLength && PageLength > 12 )
                {
			LinesPrinted = 0;
			if( !ExpertMode )
				PauseFlag = 1;
		}
        }

        if( LogFileOutput )
                if( c != '\r' )
			LogFile << c;

        if( ComPort == 0 )
        {
                if( LocalOutput )
                        local << c;
        }
        else
        {
                if( LocalOutput )
			if( ComPort == 0 || c != '\a' )
				// beep character is filtered out
                                local << c;
	
                if( RemoteOutput )
                        remote << c;
	}

        if( PauseFlag && !kin.Frozen() )
		pause();

	return c;
}

/////////////////////////////////////////////////////////////////////Local
Local::Local(): standardout()
{
}

Local::~Local()
{
}

char CC Local::_put( char c )
{
        return aout.put( c );
}

/////////////////////////////////////////////////////////////////////Remote
Remote::Remote(): standardout()
{
}

Remote::~Remote()
{
}

char CC Remote::_put( char c )
{
        if( ComPort == 0 )      return 0;

        if( ( c == '' || c == '' ) && !IBMCharacters )
                c = '|';
        if( c == '' && !IBMCharacters )
                c = '-';
        if( c == '' && !IBMCharacters )
                c = '=';
	if( (unsigned char)c > 127 && !IBMCharacters )
                c = '#';
        if( !fossil.carrierdetect() || fossil.write( c ) )
        {
                int t = ComPort;

                fossil.hangup();
                fossil.deinit();

                ComPort = 0;
		kout << "\a\nHANGUP:  COM" << t << ": could not write character\n";
                exit( 2 );
                return 0;
        }
        return c;
}

// Pause for a while (until a key is pressed)

void pause()
{
	char c;
	int t = 0;

	if( kin.Frozen() ) return;

	int SaveLogFileOutput = kout.GetLogFileOutput();
	kout.setw();
	kout.PutLogFileOutput( 0 );

	int SaveRawMode = kout.RawMode();
	kout.SetRawMode();

	if( !kin.keyhit() )
	{
		int f = kout.GetForeground(), b = kout.GetBackground();

		if( IBMCharacters )
		{
			kout << attr(PauseColor.bg) << "" << attr(PauseColor.bc,PauseColor.bg) << "PAUSED";
			kout << attr(PauseColor.bg,0) << "۲" << attr(13) << "  ";
		}
		else
			kout << attr(PauseColor.bg) << "" << attr(PauseColor.fg) << ""
			     << attr(PauseColor.bc) << "Paused" << attr(PauseColor.fg) << ""
			     << attr(PauseColor.bg) << "" << attr(13) << "  ";

		while( SpecialEffects && !kin.keyhit() && t < 10000 )
		{
			if( IBMCharacters )
			{
				kout << attr( 11 ) << "\b/";
                                multitask(); TimedDelay( 36 );
				kout << attr( 1 ) << "\bX";
                                multitask(); TimedDelay( 36 );
				kout << attr( 14 ) << "\b\\";
                                multitask(); TimedDelay( 36 );
				kout << attr( 4 ) << "\bX";
                                multitask(); TimedDelay( 36 );
			}
			else
			{
				kout << attr( 14 ) << "\b.";
                                multitask(); TimedDelay( 36 );
				kout << attr( 12 ) << "\bo";
                                multitask(); TimedDelay( 36 );
				kout << attr( 4 )  << "\bO";
                                multitask(); TimedDelay( 36 );
			}
			t++;
		}

		kout << "\b ";
		if( !kin.keyhit() ) kout << attr(15);
		kin >> c;
		kout << attr( f, b );
		for( t = 0; t < 16; t++ )
			kout << "\b \b";
	}
	else kin >> c;

	LinesPrinted = 0;

	if( !SaveRawMode ) kout.ResetRawMode();
        kout.PutLogFileOutput( SaveLogFileOutput );
}

#if TRAP_MODE

void _Cdecl signal_INT( int code )
{
	PageLength = 0;
	kout << '\n' << attr(12) << '[' << attr(14) << "INTERRUPT" << attr(12) << ']' << attr() << '\n';
	if( code == 999 ) return;
	exit( 1 );
}

void _Cdecl signal_TERM( int code )
{
        PageLength = 0;
	local << '\n' << attr(12) << '[' << attr(14) << "GAME TERMINATED" << attr(12) << ']' << attr() << '\n';
        if( code == 999 ) return;
        exit( 0 );
}

void init_trapping()
{
        signal( SIGINT, signal_INT );
        signal( SIGTERM, signal_TERM );
}

#pragma startup init_trapping

#endif TRAP_MODE

Hook InstallHook( Hook new_hook )
{
        Hook old_hook = multitask_hook;

        multitask_hook = new_hook;
        return old_hook;
}

