#ifndef SR_INOUT

#define SR_INOUT

#include "inout.h"

extern int Ansi, ComPort, ExpertMode, IBMCharacters, PageLength,
	   SpecialEffects, NoKeyTimeOut;

// Normal output and normal input colors
extern attr aNormal, aCursor;

extern char     BREAK_KEY, X_BREAK_KEY,
		STOP_KEY, X_STOP_KEY,
		CHAT_KEY, X_CHAT_KEY;

extern unsigned char SRterminal;

// Pause until a key is pressed
void pause();

// Pause Color Customization
struct _PauseColors
{
        unsigned char bg, fg, bc;       // background, foreground, bright color
};
extern _PauseColors PauseColor;

class keyin : public standardin
{
private:
	int savefore, saveback;
        int frozen;
public:
        keyin();
        ~keyin();

        void CC begininput();
        void CC endinput();

        int CC keyhit();
        char CC get();
        char CC out( char c );

        void Freeze()           { frozen = 1; }
        void Melt()             { frozen = 0; }
        int  Frozen()           { return frozen; }
};

class keyout : public standardout
{
private:
        int LogFileOpen;
        int LogFileOutput;

        friend class keyin;
public:
        int LocalOutput;
        int RemoteOutput;

        keyout();
        ~keyout();

	void CC changecolor();
	char CC _put( char c );

        void OpenLogFile( const char *filename );
        void CloseLogFile();

        void EnableLogFileOutput()      { if( LogFileOpen ) LogFileOutput = 1; }
        void DisableLogFileOutput()     { LogFileOutput = 0; }
        int  GetLogFileOutput()         { return LogFileOutput; }
        void PutLogFileOutput( int x )  { if( LogFileOpen ) LogFileOutput = x; }
};

class Local : public standardout
{
public:
        Local();
        ~Local();

	char CC _put( char c );
};

class Remote : public standardout
{
public:
        Remote();
        ~Remote();

	char CC _put( char c );
};

extern keyin kin;
extern keyout kout;
extern Local local;
extern Remote remote;

enum InputDevice { Keyboard, Modem };
extern InputDevice LastRead;

// multitask_hook is called whenever we are waiting on the user to press keys
typedef void (*Hook)(void);
extern Hook multitask_hook;
Hook InstallHook( Hook new_hook );

// keyintercept is a class that intercepts keystrokes and runs a Hook
// procedure with it.  For example, the Chat procedure creates a new
// keyintercept object as F10, but if that module is not linked in,
// the chat won't be installed.
//
// Example use:
//      (A)  To capture a key in a particular procedure (temporary)
//           void foo()
//           {
//               keyintercept F10( MyChatProcedure, 0, 68 );
//               keyintercept CtrlA( MyCtrlAProcedure, 1 );
//           }
//
//      (B)  To capture a key globally (permanent)
//               ...
//               keyintercept* F10 = new keyintercept( ChatProcedure, 0, 68 );
//               ...
//           If the procedure can occur ANY time (having no access of kin
//           and kout), you can add this:
//               F10 -> Safe();
//           and SR-COMM will run your procedure at more times.
//

class keyintercept
{
        static keyintercept* head;
               keyintercept* next;

        unsigned key;
        int anytime;
        Hook hook;
public:
        keyintercept( Hook f, unsigned k, unsigned scan = 0 );
        ~keyintercept();

        void Safe()             { anytime = 1; }
        void UnSafe()           { anytime = 0; }

        // These take a key and search for the procedure to run; anytime = 0
        // will run any Safe key; anytime = 1 will run any key
        static Hook Search( unsigned k, int anytime = 0 );
        static Hook SearchExtended( unsigned scancode, int anytime = 0 );
};

#endif

